# Windows PwowerShell
#---------------------------------------------------------
#
# RR-2 MQI Client Bindings (for Responder)
#
# Parameters:
# $1 : Queue manager name
# $2 : Number of connections
# $3 : Messages
# $4 : Message Bytes
# $5 : Log Directory
# $6 : Workload
# $7 : MQ Constant
# $8 : Data File
#
# Environments(./runpfmts.bat):
# $INCL_CON_TIME : Include MQCONN/MQDISC time in measurement,
#                  0:not include, 1:include
# $RCV_BUFF_SIZE : Receive buffer size
# $CON_RETRY     : The number of connection retry
# $CON_INTERVAL  : Connection retry interval
# $RR2_RequestQ  : Request queue name
# $RR2_ReplyQ    : Reply queue name
#
# History
#
# Date        Create/Update           Description
# 2022.02.21  Pulsar Integration Inc. Initial Version
#
#---------------------------------------------------------
param(
    [string]$TARGET_QMGR = "",
    [int]$Connections = 0,
    [int]$Messages = 0,
    [int]$Bytes = 0,
    [string]$LogDirPre = "",
    [string]$Workload = "",
    [string]$MQ_Constant = "",
    [string]$DataFile = ""
)

# return code
[int]$RET_OK=0
[int]$RET_NG=1

$USAGE = $myInvocation.MyCommand.name + " <Qmgr> <Connections> <Messages> <Bytes> <Log Directory> <Workload> <MQ Constant> <Data File>"

if ($PSBoundParameters.Count -lt 6)
{
    Write-output "" "Usage: $USAGE" "" ""

    $usage_document = @"
    RR-2 MQI Client Bindings (for Responder)

    Parameters:
    `$1 : Queue manager name
    `$2 : Number of connections
    `$3 : Number of messages
    `$4 : Message Bytes
    `$5 : Log Directory
    `$6 : Workload
    `$7 : MQ Constant (ex. MQPER_PERSISTENT, MQPER_NOT_PERSISTENT)
    `$8 : Data File (Input message file) (Option)
         If not specified, "<workload name>" is used as the message.

    Environments(./runpfmts.bat):
    `$INCL_CON_TIME : Include MQCONN/MQDISC time in measurement,
                     0:not include, 1:include
    `$RCV_BUFF_SIZE : Receive buffer size
    `$CON_RETRY     : The number of connection retry
    `$CON_INTERVAL  : Connection retry interval
    `$RR2_RequestQ  : Request queue name
    `$RR2_ReplyQ    : Reply queue name

"@

    Write-output $usage_document

    exit $RET_NG
}

# import library
. ./lib/libpfmts.ps1

# source environments for performance test
fnSourceEnvironments

[string]$RR2_RequestQ=${env:RR2_RequestQ}
[string]$RR2_ReplyQ=${env:RR2_ReplyQ}
[int]$CON_RETRY=${env:CON_RETRY}
[int]$CON_INTERVAL=${env:CON_INTERVAL}
[int]$RCV_BUFF_SIZE=${env:RCV_BUFF_SIZE}
[int]$INCL_CON_TIME=${env:INCL_CON_TIME}

$TRM = ""
if ($INCL_CON_TIME -eq 1)
{
    $TRM = "-tr"
}

$dateTime = Get-Date -Format "yyyyMMdd_HHmmss"
$LogDir = "${LogDirPre}\${Workload}_${Bytes}_${Connections}_${TARGET_QMGR}_${dateTime}"

mkdir $LogDir

$LogNamePre = "${LogDir}\rep"
$StderrLogNamePre = "${LogDir}\stderr_rep"

Write-Host ""
Write-Host "TARGET_QMGR: $TARGET_QMGR"
Write-Host "Connections: $Connections Messages: $Messages Bytes: $Bytes LogDirPre: $LogDirPre Workload: $Workload MQ_Constant: $MQ_Constant DataFile: $DataFile"

$Total_Messages = $Connections * $Messages

$WaitProcess = @()

for ( $Counter = 1; $Counter -le $Connections; $Counter++ )
{
    # Responder
    # primary side: MQGMO_ACCEPT_TRUNCATED_MSG
    # secondary side: -l $Bytes

    if ($DataFile -eq "")
    {
        $ArgumentList = "-qm $TARGET_QMGR -q $RR2_RequestQ -oq $RR2_ReplyQ MQGMO_WAIT MQWI_UNLIMITED MQGMO_NO_SYNCPOINT MQGMO_ACCEPT_TRUNCATED_MSG -n $Messages -cr $CON_RETRY -wp $CON_INTERVAL -sz $RCV_BUFF_SIZE $TRM -ss -l $Bytes MQMT_REPLY $MQ_Constant MQPMO_NO_SYNCPOINT -im"
    }
    else
    {
        $ArgumentList = "-qm $TARGET_QMGR -q $RR2_RequestQ -oq $RR2_ReplyQ MQGMO_WAIT MQWI_UNLIMITED MQGMO_NO_SYNCPOINT MQGMO_ACCEPT_TRUNCATED_MSG -n $Messages -cr $CON_RETRY -wp $CON_INTERVAL -sz $RCV_BUFF_SIZE $TRM -ss -l $Bytes MQMT_REPLY $MQ_Constant MQPMO_NO_SYNCPOINT -im -f $DataFile"
    }

    $dateTime = Get-Date -Format "yyyyMMdd_HHmmss"
    $logName = "${LogNamePre}_${Counter}_${dateTime}.log"
    $stderrLogName = "${StderrLogNamePre}_${Counter}_${dateTime}.log"

    Write-Host "Start Responder Program, Get $RR2_RequestQ, Put $RR2_ReplyQ"
    Write-Host "mqpgf $ArgumentList"

    $WaitProcess += Start-Process -FilePath mqpgf.exe -RedirectStandardOutput $logName -RedirectStandardError $stderrLogName -PassThru -NoNewWindow -ArgumentList $ArgumentList
    Write-Host "start background: $Counter of $Connections."
}

$index = 0
for ( ; $index -lt $WaitProcess.count; $index++)
{
    do
    {
        try
        {
            $process = Get-Process -Id $WaitProcess[$index].Id -ErrorAction stop | Format-Table
            Write-Output "The number of the sampling process displayed: $index"
            $process
        }
        catch
        {
            Write-Output "The end of the test command has been detected. Process Number: $index"
            break
        }
        Start-Sleep 5
    }
    while(1)
}

Write-Host ""
fnGetElapsedTime -LOGFILE_PREFIX "$LogNamePre" -MESSAGES $Messages -CONNECTIONS $Connections | tee-object ${LogDirPre}/console.log -Append

exit $RET_OK
